/*
    Copyright (C) 2014-2019 de4dot@gmail.com

    This file is part of dnSpy

    dnSpy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dnSpy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dnSpy.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.ComponentModel.Composition;
using System.Windows.Media;
using System.Windows.Media.TextFormatting;

namespace dnSpy.Text.Formatting {
	[Export(typeof(ITextFormatterProvider))]
	sealed class TextFormatterProvider : ITextFormatterProvider {
		readonly TextFormatter idealModeTextFormatter;
		readonly TextFormatter displayModeTextFormatter;

		[ImportingConstructor]
		TextFormatterProvider() {
			idealModeTextFormatter = TextFormatter.Create(TextFormattingMode.Ideal);
			displayModeTextFormatter = TextFormatter.Create(TextFormattingMode.Display);
		}

		// These formatters are only used on the UI thread so they can be shared by all callers
		public TextFormatter Create(bool useDisplayMode) => useDisplayMode ? displayModeTextFormatter : idealModeTextFormatter;
	}
}
